/* by Ola Sigurdson */

#include <sched.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>

const long MemIncr= 	64L*1024*1024;
const int killtime= 	30;

int main () {
  printf ("I am a typical greedy application that will use all memory and CPU\n");

  long processors=sysconf(_SC_NPROCESSORS_ONLN);

  printf ("Creating %ld threads that will eat all memory in %ld byte increments\n", processors, MemIncr);
  printf ("All threads will automatically be killed after %d seconds\n", killtime);
  printf ("(If the parent process survives and gets CPU time...)\n");
  fflush (stdout);
  
  for (long i= 0 ; i < processors; i++) {
    /* Use fork to make resource limiting a bit more challenging */
    pid_t p= fork();

    if (p < 0) {
      perror ("Thread creation failed");
      return errno;
    } else if (p > 0) {
      printf ("Started PID %ld\n", p);
      fflush (stdout);      
    } else if (p == 0) {
      usleep (100*1024);
      size_t s= 0;
      int *mem=NULL;
      int ok=1;
      
      while (ok) {																														sleep (0); /* yield */
	void * r=realloc (mem, s+ MemIncr);
	ok= (NULL != r);
	if (ok) {
	  mem= r;
	  s += MemIncr;
//	  printf ("Allocated %ld bytes\n", s);
	}	  
      }
      if (NULL == mem) {
	printf ("No memory allocated, exiting\n");
	exit (-1);
      }
      printf ("Allocated %ld bytes, using them\n", s);
      fflush (stdout);
      usleep (100*1024);
      /* Touch it */
      while (1) {
	for (long i= 0;
	    i < (s / sizeof(int));  
	    i ++) 
	{
	  mem[i]= (int)i;
	}
      }
      return 0;
    }

  }
  sleep (killtime);
  printf ("Reached end of runtime, killing all threads.\n");
  fflush (stdout);
  kill (0, SIGKILL);
  return 0;
}
